<?php


function last_char($string) {
  // return the last character of a string
  $last_char = null;

  if ($string) {
    $last_char = $string[(strlen($string) -1)];
  }
  return $last_char;
}


function strpre($haystack, $needle) {
  // returns portion of haystack that exists behind first occurrence of needle

  $end_position = strpos($haystack, $needle);
  $fin_string = str_split($haystack, $end_position);  
  return $fin_string[0];
}


function bck_remove($string) {
  // assumes that string is a filename or dirpath
  // if there are occurences of '../' in the dirpath, these are removed before
  // the dirpath is returned

  if ($string) {
    // if first three chars of string '../'
    // remove '../' and return rest of string
    $new_string = $string;
    
    $first_chars = str_split($string, 3);
    if ($first_chars[0] == '../') {
      $new_string = substr($string, 2);
      $new_string = bck_remove($new_string);
    }
  }
  return $new_string;
}


function items_in_string($array_of_items, $string) {    
  // returns $string, if it is found in array of strings,
  // returns null if $string is not found

  $in_string = null;
  for($x=0;$x<count($array_of_items);$x++) {
    if (strpos($string, $array_of_items[$x])) {
      $in_string = $array_of_items[$x];
    }
    if (strpos($string, strtoupper($array_of_items[$x]))) {
      $in_string = $array_of_items[$x];
    }
  }
  return $in_string;
}


function str_remove($item, $string) {
  // removes occurence of $item in $string and returns modified $string

  return str_replace($item, '', $string);
}


function decode_vars($vars) {
  // assumes $vars is a string. decodes sequences in the variable and reutrns
  // a new string with decoded sequences replacing the encoded ones.

  // note: this function should be modified to include parameters for decoding, 
  // -translate 'foo' into 'bar'
  // currently, it decodes 'E' to '=' and 'A' to '&'
  // new parameters should include variable width array for flexibility

  $decoded_vars = null;
  $decoded_vars = str_replace('E', '=', $vars);
  $decoded_vars = str_replace('A', '&', $decoded_vars);
  return $decoded_vars;
}


function decode_dirs($dirs) {
  // assumes $vars is a string. decodes sequences in the variable and reutrns
  // a new string with decoded sequences replacing the encoded ones.

  $decoded_dirs = null;
  $decoded_dirs = str_replace('Q', '/', $dirs);
  $decoded_dirs = str_replace('D', '.', $decoded_dirs);
  
  // if first char not dir slash, append dir slash
  if ($decoded_dirs[0] != '/') {
    $decoded_dirs = '/'.$decoded_dirs;
  }
  
  return $decoded_dirs;
}

function encode_vars($vars) {
  // assumes $vars is a string. encodes sequences in the variable and returns
  // a new string with encoded sequences replacing the un-encoded ones.

  $encoded_vars = null;
  $encoded_vars = str_replace('=', 'E', $vars);
  $encoded_vars = str_replace('&', 'A', $encoded_vars);
  return $encoded_vars;
}

function encode_dirs($dirs) {
  // assumes $vars is a string. encodes sequences in the variable and returns
  // a new string with encoded sequences replacing the un-encoded ones.

  $encoded_dirs = null;
  $encoded_dirs = str_replace('/', 'Q', $dirs);
  $encoded_dirs = str_replace('.', 'D', $encoded_dirs);
  return $encoded_dirs;
}

function path_encode($string) {
  // takes $string and routes it through encode_vars and encode_dir, then
  // returns the encoded string in the format ?P=$encoded_dirs&V=$encoded_vars
  // so that it may be appended to the end of a url
  
  // note, function should be re-written for flexibility...

  if ($string) {
    if (strpos($string, '?')) {
      $temp_vars = strstr($string, '?');
      $temp_path = strpre($string, '?');
    } else {
      $temp_vars = null;
      $temp_path = $string;
    }
  }
  
  if ($temp_vars) {
    $encoded_vars = encode_vars($temp_vars);
  }
  
  if ($temp_path) {
    $encoded_path = encode_dirs($temp_path);
  }
 
  $string_mark = 0;
  $dir_string = null;
  if ($encoded_path) {
    $dir_string = "P=".$encoded_path;
    $mark_string += 1;
  }
  
  $var_string = null;
  if ($encoded_vars) {
    $var_string = "V=".$encoded_vars;
    $mark_string += 3;
  }
  
  switch($mark_string) {
    case 0:
      $new_string = null;
      break;
    case 1:
      $new_string = "?"."P=".$encoded_path;
      break;
    case 3:
      $new_string = "?"."V=".$encoded_vars;
      break;
    case 4:
      $new_string = "?"."P=".$encoded_path."&"."V=".$encoded_vars;
      break;
    default:
      $new_string = null;
  }

  return $new_string;  
}


function get_url_following_delimeter($string, $delimeter) {
  // if url follows delimeter 
  // - returns the url following the delimeter,
  // else
  // - returns null

  // $string must be a valid html string
  
  $sub_string_str = null;
  
  // explode string around whitespace
  $string_array = split(' ', $string); 
  $foc_string_int = index_of_elt_w_substr($string_array, $delimeter);
  $foc_string_str = $string_array[$foc_string_int];
  // foc_string_str might be: href="/p/gal.php?p=1">
  
  // split substring arount '"' characters
  $substr_array = split('"', $foc_string_str);
  $sub_string_int = 1; 
  $sub_string_str = $substr_array[$sub_string_int];
  // sub_string_str might be: /p/ga.php?p=1
    
  return $sub_string_str;
}


function html_append($string, $delimeter, $toappend, $cancelstr) {
  // expands a string and encodes string in specified delimeter 
  // example: href="string" -> href="encodedstring"  
  // /p/ga.php?p=1&r=3 ---> P=QpQgaDphp&V=pE1ArE3
  
  // if $toappend is found in $string, $string is returned
  // unmodified
  
  // if 'http://' if found in $string, $string is returned
  // unmodified
  
  
  $encoded_path = null;
  $encoded_vars = null;

  $temp_path = null;
  $temp_vars = null;

  $append = 1;

  // explode string around whitespacc
  $string_array = split(' ', $string); 
  $foc_string_int = index_of_elt_w_substr($string_array, $delimeter);
  $foc_string_str = $string_array[$foc_string_int];
  // foc_string_str might be: href="/p/gal.php?p=1">
  
  // split substring arount '"' characters
  $substr_array = split('"', $foc_string_str);
  $sub_string_int = 1; 
  $sub_string_str = $substr_array[$sub_string_int];
  // sub_string_str might be: /p/ga.php?p=1
  
  if (is_numeric(strpos($sub_string_str, 'http://'))) {
    $append = 0;
  }
  
  if (is_numeric(strpos($sub_string_str, 'https://'))) {
    $append = 0;
  }

  if (is_numeric(strpos($sub_string_str, $cancelstr))) {
    $append = 0;
  }
  
  $cancelstr = str_replace("www.", "", $cancelstr);
  
  if (is_numeric(strpos($sub_string_str, $cancelstr))) {
    $append = 0;
  }
  
  if ($append == 0) { 
    $substr_array[$sub_string_int] = $sub_string_str;
  } else {
    //if sub string does not have dir slash
    $noslash = 0;

    if ($sub_string_str[0] == '/') {
      $noslash += 1;
    }
    
    if ($sub_string_str[0] == '?') {
      $noslash += 1;
    }
    
    if (last_char($toappend) == '/') {
      $noslash += 1;
    }
    
    if ($noslash == 0) {
      $sub_string_str = '/'.$sub_string_str;
    }

    $substr_array[$sub_string_int] = $toappend.$sub_string_str;
  }

  // put string together...
  unset($foc_string_str);
  $foc_string_str = $substr_array[0];
  for($x=1;$x<count($substr_array);$x++) {
    $foc_string_str = $foc_string_str.'"'.$substr_array[$x];
  }

  // foc_string_str might be: href="/p/gal.php?p=1">
  $string_array[$foc_string_int] = $foc_string_str;

  unset($string);
  $string = $string_array[0];
  for($x=1;$x<count($string_array);$x++) {
    $string = $string.' '.$string_array[$x];
  }

  return $string;
}


function html_encode($string, $delimeter) {  
  // expands a string and encodes string in specified delimeter 
  // example: 
  // href="string"
  // becomes
  // href="encodedstring"  

  // /p/ga.php?p=1&r=3 ---> P=QpQgaDphp&V=pE1ArE3

  $encoded_path = null;
  $encoded_vars = null;

  $temp_path = null;
  $temp_vars = null;

  // explode string around whitespacc
  $string_array = split(' ', $string); 
  $foc_string_int = index_of_elt_w_substr($string_array, $delimeter);
  $foc_string_str = $string_array[$foc_string_int];
  // foc_string_str might be: href="/p/gal.php?p=1">
  
  // split substring arount '"' characters
  $substr_array = split('"', $foc_string_str);
  $sub_string_int = 1; 
  $sub_string_str = $substr_array[$sub_string_int];
  // sub_string_str might be: /p/ga.php?p=1

  $append = 1;
  if (is_numeric(strpos($sub_string_str, 'http://'))) {
    $append = 0;
  }

  if (is_numeric(strpos($sub_string_str, 'https://'))) {
    $append = 0;
  }
  
  if ($append == 1) {
    $sub_string_str = path_encode($sub_string_str);
  } else {
    $sub_string_str = $sub_string_str;
  }

  // put string together...
  $substr_array[$sub_string_int] = $sub_string_str;

  unset($foc_string_str); // clear foc_string
  $foc_string_str = $substr_array[0];
  for($x=1;$x<count($substr_array);$x++) {
    $foc_string_str = $foc_string_str.'"'.$substr_array[$x];
  }

  // foc_string_str might be: href="/p/gal.php?p=1">
  $string_array[$foc_string_int] = $foc_string_str;

  unset($string);
  $string = $string_array[0];
  for($x=1;$x<count($string_array);$x++) {
    $string = $string.' '.$string_array[$x];
  }

  return $string;
}


function index_of_elt_w_substr($array, $substr) {
  // return the index of the first element containing the substring

  $index_of_elt = null;
  for($x=0;$x<count($array);$x++) {
    if (is_numeric(strpos($array[$x], $substr))) {
      $index_of_elt = $x; 
    }
    // check for upper case instance
    if (is_numeric(strpos($array[$x], strtoupper($substr)))) {
      $index_of_elt = $x;
    }
  }
  return $index_of_elt;
}


function set_to_array($item) {
  // if $item is not array, push $item to array and return array
  // else return $item

  $value = array();
  if (is_array($item)) {
    $value = $item;
  } else {
    array_push($value, $item);
  }
  return $value;
}


function write_array_to_file($array, $filename) {
  // writes the lines in $array to the file $filename

  if (file_exists($filename)) {
    echo "creating ".$filename.".";
  }

  $handle = fopen($filename, "wb");
  foreach($array as $line) {
    $line = $line."\n";
    fwrite($handle, $line);
  }

  fclose($handle);
}

function replace_chars($string, $remchar, $addchar) {
  // replace occurences of $remchar with $addchar in $string

  $new_string = null;
  $tmp_string = array();

  if (strstr($string, $remchar)) {
    $temp_string = split($remchar, $string);
    foreach($temp_string as $token) {
      $new_string = $addchar.$token.$new_string;    
    }
  } else {
    $new_string = $string;
  }
  return $new_string;
}



function identify_lines_in_tag($html_array, $tag) {
  // <body ... >
  // </body>

  $lines = array();
  $bgn_tag = "<".strtolower($tag);
  $end_tag = "</".strtolower($tag);
  $trigger = 0;

  for($x=0;$x<count($html_array);$x++) {
    $token = $html_array[$x];

    if (strstr(strtolower($token), $end_tag)) {
       $trigger = 0;
    }
    if ($trigger == 1) {
      array_push($lines, $x);
    }
    if (strstr(strtolower($token), $bgn_tag)) {
      $trigger = 1;
    }
  }
  return $lines;
}

  
function get_files($directory) {
  // returns all files (full path included) from $directory
  // and all sub directories
  
  // this is a slightly modified script originally found on the php doc pages

  // open directory
  if($dir = opendir($directory)) {
    $tmp = Array(); // array to store files
    // Add the files
    while($file = readdir($dir)) {
      // Make sure the file exists
      if($file != "." && $file != ".." && $file[0] != '.') {
        // If it's a directiry, list all files within it
        if(is_dir($directory."/".$file)) {
          $tmp2 = get_files($directory."/".$file);
          if(is_array($tmp2)) {
            $tmp = array_merge($tmp, $tmp2);
          }
        } else {
          array_push($tmp, $directory."/".$file);
        }
      }
    }
    // Finish off the function
    closedir($dir);
    return $tmp;
  }
}

?>
